@@ -18,8 +18,9 @@ L<Catalyst::Controller> subclasses.
 =cut
 
 use Moose;
-
+use Scalar::Util 'looks_like_number';
 with 'MooseX::Emulate::Class::Accessor::Fast';
+use namespace::clean -except => 'meta';
 
 has class => (is => 'rw');
 has namespace => (is => 'rw');
@@ -28,8 +29,6 @@ has attributes => (is => 'rw');
 has name => (is => 'rw');
 has code => (is => 'rw');
 
-no Moose;
-
 use overload (
 
     # Stringify to reverse for debug output etc.
@@ -70,6 +69,18 @@ sub match {
     return scalar( @{ $c->req->args } ) == $args;
 }
 
+sub compare {
+    my ($a1, $a2) = @_;
+
+    my ($a1_args) = @{ $a1->attributes->{Args} || [] };
+    my ($a2_args) = @{ $a2->attributes->{Args} || [] };
+
+    $_ = looks_like_number($_) ? $_ : ~0 
+        for $a1_args, $a2_args;
+
+    return $a1_args <=> $a2_args;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -105,6 +116,11 @@ context and arguments
 Check Args attribute, and makes sure number of args matches the setting.
 Always returns true if Args is omitted.
 
+=head2 compare
+
+Compares 2 actions based on the value of the C<Args> attribute, with no C<Args>
+having the highest precedence.
+
 =head2 namespace
 
 Returns the private namespace this action lives in.
@@ -127,7 +143,7 @@ Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
-This program is free software, you can redistribute it and/or modify it under
+This library is free software. You can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
